using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using FakeItEasy;
using FluentEmail.Core;
using FluentEmail.Core.Defaults;
using FluentEmail.Core.Interfaces;
using FluentEmail.Core.Models;
using NUnit.Framework;

namespace StockAdvisory.Tests;

public class StockEmailSenderTests
{
    private ISender _sender = A.Fake<ISender>();
    private IFluentEmailFactory _factory = A.Fake<IFluentEmailFactory>();
    private IStockFormatter _formatter = A.Fake<IStockFormatter>();
    private StockEmailSender _stockSender;
    
    
    [SetUp]
    public void SetUp()
    {
        _factory = A.Fake<IFluentEmailFactory>();
        _sender = A.Fake<ISender>();
        _formatter = A.Fake<IStockFormatter>();
        A.CallTo(() => _factory.Create())
            .ReturnsLazily(_ => new Email(new ReplaceRenderer(), _sender));
        A.CallTo(() => _formatter.Format(A<IEnumerable<StockDifferenceRecord>>._))
            .Returns("Mock format");
        _stockSender = new StockEmailSender(_factory, _formatter);
    }

    public void SendNoErrorsOnSend()
    {
        A.CallTo(() => _sender.SendAsync(A<IFluentEmail>.Ignored, A<CancellationToken?>.Ignored))
            .WithAnyArguments()
            .Returns(Task.FromResult(new SendResponse())); 
        A.CallTo(() => _sender.Send(A<IFluentEmail>.Ignored, A<CancellationToken?>.Ignored))
            .WithAnyArguments()
            .Returns(new SendResponse());
    }
    
    
    [Test]
    public async Task Test_When_SubscriberEmailsAndRecordsArePassed_Then_EmailsGetSent()
    {
        SendNoErrorsOnSend();

        var records = new List<StockDifferenceRecord>() {new StockDifferenceRecord()};
        var clients = new List<string>() {"t@e.sting", "sample@email"};

        await _stockSender.SendReports(clients, records);
        
        // Makes sure that two emails were sent, each separately for each address
        A.CallTo(() =>
                _sender.SendAsync(
                    A<IFluentEmail>.That.Matches(e =>
                        e.Data.ToAddresses.Select(a => a.EmailAddress).Intersect(clients).Count() == 1),
                    A<CancellationToken?>.Ignored))
            .MustHaveHappenedTwiceExactly();
    }

    [Test]
    public async Task Test_WhenNoStockDifferencesArePassed_Then_NoEmailsAreSent()
    {
        SendNoErrorsOnSend();
        var clients = new List<string>() {"t@e.sting", "sample@email"};
        var records = new List<StockDifferenceRecord>();

        A.CallTo(() => _formatter.Format(A<IEnumerable<StockDifferenceRecord>>._))
            .Returns(String.Empty);
        await _stockSender.SendReports(clients, records);
        
        A.CallTo(() =>
            _sender.SendAsync(A<IFluentEmail>._, A<CancellationToken>._))
            .WithAnyArguments()
            .MustNotHaveHappened();
    }
}