using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using NUnit.Framework;

namespace StockAdvisory.Tests;

public class StockTextFormatterTests
{
    public StockEntryRecord TeslaRecord = new StockEntryRecord()
    {
        Date = new DateTime(2022, 3, 28), Company = "TESLA INC", Ticker = "TSLA",
        CuspId = "88160R101", Shares = 1_166_661, MarketValue = 1_179_074_273.04M, Weight = 100.00M
    };
    
    public StockDifferenceRecord SpotifyDiffRecord = new StockDifferenceRecord()
    {
        Company = "SPOTIFY TECHNOLOGY SA", Ticker = "SPOT", CuspId = "L8681T102",
        Shares = 3_229_198, SharesPercentChange = (float) 4.01, Weight = 504_271_559
    };

    [Test]
    public void Test_When_SingleStockEntryRecordIsProvided_Then_AllRelevantInformationIsContained()
    {
        var formatter = new StockTextFormatter();
        var formattedString = formatter.Format(TeslaRecord);
        StringAssert.Contains(TeslaRecord.Company, formattedString);
        StringAssert.Contains(TeslaRecord.Ticker, formattedString);
        StringAssert.Contains(TeslaRecord.Shares.ToString(), formattedString);
        StringAssert.Contains(TeslaRecord.Weight.ToString(CultureInfo.InvariantCulture),
            formattedString);
        // what is contained is up for debate, right now everything except date
    }

    [Test]
    public void Test_When_CollectionWithSingleStockEntryIsProvided_Then_AllRelevantInformationIsContained()
    {
        var formatter = new StockTextFormatter();
        var records = new List<StockEntryRecord>() {TeslaRecord};
        var formattedString = formatter.Format(records);
        StringAssert.Contains(TeslaRecord.Company, formattedString);
        StringAssert.Contains(TeslaRecord.Ticker, formattedString);
        StringAssert.Contains(TeslaRecord.Shares.ToString(), formattedString);
        StringAssert.Contains(TeslaRecord.Weight.ToString(CultureInfo.InvariantCulture),
            formattedString);
        // what is contained is up for debate, right now everything except date 
    }

    [Test]
    public void Test_When_CollectionOfStockEntriesIsProvided_Then_OutputDoesNotStartWithEmptyLine()
    {
        var formatter = new StockTextFormatter();
        var records = new List<StockEntryRecord>() {TeslaRecord};
        var formattedString = formatter.Format(records);
        var stringReader = new StringReader(formattedString);
        var line = stringReader.ReadLine();
        Assert.NotNull(line);
        Assert.NotZero(line!.Trim().Length);
    }

    [Test]
    public void Test_When_StockDifferenceRecordIsProvided_Then_AllRelevantInformationIsContained()
    {
        var formatter = new StockTextFormatter();
        var formattedString = formatter.Format(SpotifyDiffRecord);
        StringAssert.Contains(SpotifyDiffRecord.Company, formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Ticker, formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Shares.ToString(), formattedString);
        StringAssert.Contains(SpotifyDiffRecord.SharesPercentChange.ToString(CultureInfo.InvariantCulture), 
            formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Weight.ToString(CultureInfo.InvariantCulture),
            formattedString);
    }
    
    [Test]
    public void Test_When_CollectionWithSingleStockDiffRecordIsProvided_Then_AllRelevantInformationIsContained()
    {
        var formatter = new StockTextFormatter();
        var records = new List<StockDifferenceRecord>() { SpotifyDiffRecord };
        var formattedString = formatter.Format(records);
        StringAssert.Contains(SpotifyDiffRecord.Company, formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Ticker, formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Shares.ToString(), formattedString);
        StringAssert.Contains(SpotifyDiffRecord.SharesPercentChange.ToString(CultureInfo.InvariantCulture),
            formattedString);
        StringAssert.Contains(SpotifyDiffRecord.Weight.ToString(CultureInfo.InvariantCulture),
            formattedString);
    }
}